/*==LICENSE==*

CyanWorlds.com Engine - MMOG client, server and tools
Copyright (C) 2011  Cyan Worlds, Inc.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Additional permissions under GNU GPL version 3 section 7

If you modify this Program, or any covered work, by linking or
combining it with any of RAD Game Tools Bink SDK, Autodesk 3ds Max SDK,
NVIDIA PhysX SDK, Microsoft DirectX SDK, OpenSSL library, Independent
JPEG Group JPEG library, Microsoft Windows Media SDK, or Apple QuickTime SDK
(or a modified version of those libraries),
containing parts covered by the terms of the Bink SDK EULA, 3ds Max EULA,
PhysX SDK EULA, DirectX SDK EULA, OpenSSL and SSLeay licenses, IJG
JPEG Library README, Windows Media SDK EULA, or QuickTime SDK EULA, the
licensors of this Program grant you additional
permission to convey the resulting work. Corresponding Source for a
non-source form of such a combination shall include the source code for
the parts of OpenSSL and IJG JPEG Library used as well as that of the covered
work.

You can contact Cyan Worlds, Inc. by email legal@cyan.com
 or by snail mail at:
      Cyan Worlds, Inc.
      14617 N Newport Hwy
      Mead, WA   99021

*==LICENSE==*/
/*****************************************************************************
*
*   $/Plasma20/Sources/Plasma/NucleusLib/pnUtils/Private/Win32/pnUtW32Addr.cpp
*   
***/

#include "../pnUtils.h"


/*****************************************************************************
*
*   Internal functions
*
***/

//===========================================================================
static NetAddressNode NodeFromString (const wchar_t * string[]) {
    // skip leading whitespace
    const wchar_t * str = *string;
    while (iswspace(*str))
        ++str;

    // This function handles partial ip addresses (61.33)
    // as well as full dotted quads. The address can be
    // terminated by whitespace or ':' as well as '\0'
    uint8_t data[4];
    * (uint32_t *) data = 0;
    for (unsigned i = sizeof(data); i--; ) {
        if (!iswdigit(*str))
            return (unsigned)-1;

        unsigned value = StrToUnsigned(str, &str, 10);
        if (value >= 256)
            return (unsigned)-1;
        data[i] = (uint8_t) value;

        if (!*str || (*str == ':') || iswspace(*str))
            break;

        static const wchar_t s_separator[] = L"\0...";
        if (*str++ != s_separator[i])
            return (unsigned)-1;
    }

    *string = str;
    return * (NetAddressNode *) &data[0];
}


/*****************************************************************************
*
*   Exports
*
***/

//===========================================================================
int NetAddressCompare (const NetAddress & a1, const NetAddress & a2) {
    const sockaddr_in & i1 = * (const sockaddr_in *) &a1;
    const sockaddr_in & i2 = * (const sockaddr_in *) &a2;

    int   d = i1.sin_addr.S_un.S_addr - i2.sin_addr.S_un.S_addr;
    return d ? d : i1.sin_port - i2.sin_port;
}

//===========================================================================
void NetAddressToString (
    const NetAddress &  addr, 
    wchar_t *             str, 
    unsigned            chars, 
    ENetAddressFormat   format
) {
    ASSERT(str);

    static const wchar_t * s_fmts[] = {
        L"%S",      // kNetAddressFormatNodeNumber
        L"%S:%u",   // kNetAddressFormatAll
    };
    ASSERT(format < arrsize(s_fmts));
    const sockaddr_in & inetaddr = * (const sockaddr_in *) &addr;
    StrPrintf(
        str,
        chars,
        s_fmts[format],
        inet_ntoa(inetaddr.sin_addr),
        ntohs(inetaddr.sin_port)
    );
}

//===========================================================================
bool NetAddressFromString (NetAddress * addr, const wchar_t str[], unsigned defaultPort) {
    ASSERT(addr);
    ASSERT(str);

    // NetAddress is bigger than sockaddr_in so start by zeroing the whole thing
    memset(addr, 0, sizeof(*addr));
    
    for (;;) {
        NetAddressNode node = NodeFromString(&str);
        if (node == (unsigned)-1)
            break;

        if (*str == L':')
            defaultPort = StrToUnsigned(str + 1, nil, 10);
   
        sockaddr_in * inetaddr          = (sockaddr_in *) addr;
        inetaddr->sin_family            = AF_INET;
        inetaddr->sin_port              = htons((uint16_t) defaultPort);
        inetaddr->sin_addr.S_un.S_addr  = htonl(node);
        // inetaddr->sin_zero already zeroed

        return true;
    }

    // address already zeroed
    return false;
}

//===========================================================================
unsigned NetAddressGetPort (
    const NetAddress & addr
) {
    return ntohs(((sockaddr_in *) &addr)->sin_port);
}

//===========================================================================
void NetAddressSetPort (
    unsigned        port,
    NetAddress *    addr
) {
    ((sockaddr_in *) addr)->sin_port = htons((uint16_t) port);
}

//============================================================================
NetAddressNode NetAddressGetNode (const NetAddress & addr) {
    return ntohl(((const sockaddr_in *) &addr)->sin_addr.S_un.S_addr);
}

//===========================================================================
void NetAddressFromNode (
    NetAddressNode  node,
    unsigned        port,
    NetAddress *    addr
) {
    memset(addr, 0, sizeof(*addr));
    sockaddr_in * inetaddr          = (sockaddr_in *) addr;
    inetaddr->sin_family            = AF_INET;
    inetaddr->sin_addr.S_un.S_addr  = htonl(node);
    inetaddr->sin_port              = htons((uint16_t) port);
}

//===========================================================================
void NetAddressNodeToString (
    NetAddressNode  node,
    wchar_t *         str,
    unsigned        chars
) {
    in_addr addr;
    addr.S_un.S_addr = htonl(node);
    StrPrintf(str, chars, L"%S", inet_ntoa(addr));
}
